#include "microdelay.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/io.h>
#include <sys/types.h>
#include <fcntl.h>

#define inc_bit    0x01
#define vpp_bit    0x02
#define prog_bit   0x04 /* not inverse */
#define erase_bit  0x08 
#define prog_mode  0x0F
#define erase_mode 0x07
#define idle       0x09 /* Idle, rst = 0 */

#define MEM_SIZE 2048

int main(int argc, char **argv) {

  // parallel programmer variables
  unsigned int port=0x378;
  unsigned char mem[MEM_SIZE];
  
  int i, j, k, l, fd;
  

  /* check root privileges */
  if (geteuid()!=0) 
    {
      fprintf(stderr,"ERROR: this programs needs root permisions to access "
	      "parallel port. Please use root user to run or setuid bit.\n");
      abort();
    }
  ioperm(port,3,1);

  microdelay_init(); //initialize microdelay function

  for (i = 0; i < MEM_SIZE; i++)
	mem[i] = 0xFF;


  fd = open(argv[1], O_RDONLY);
  if (!fd)
	{
	  perror(argv[1]);
	  outb(idle, port+2);
	  outb(0x00, port);
	  exit(2);
	}

  read(fd, mem, MEM_SIZE);
  close(fd);

#define nms(x) microdelay((x)*1000)
  

  /* Erase */
  outb(0xFF, port);
  outb(0x01, port+2);
  nms(10);
  outb(0x03, port+2); /* RST = 12 */
  nms(1);
  outb(erase_mode, port+2);
  nms(1);
  outb(erase_mode^prog_bit, port+2); /* P3.2 = 0 */
  nms(12);
  outb(erase_mode, port+2); /* P3.2 = 1 */
  nms(12);
  outb(erase_mode ^ vpp_bit, port+2); /* RST = 0 */
  nms(20);
  

  /* Program FLASH */
  outb(idle, port+2);
  nms(10);
  outb(idle | vpp_bit, port+2);
  nms(10);
  outb(prog_mode, port+2); /* RST = 12, PROG = H */
  nms(1);
  printf("Blowing");
  k = 0;
  l = 0;
  for(j = 0; j < MEM_SIZE; j++)
    {
      outb(mem[j], port);
      if (mem[j] != 0xFF)
	{
	  microdelay(2);
	  /* PROG pulse */
	  outb(prog_mode ^ prog_bit, port+2); /* P3.2 = 0 */
	  microdelay(5);
	  outb(prog_mode, port+2); /* P3.2 = 1 */
	  i = 0;
	  while ( (inb(port+1) & 64) == 0)
	    {
	      microdelay(20);
	      if (i > 200)
		{
		  printf("Error, never ready?\n");
		  outb(idle, port+2);
		  outb(0x00, port);
		  exit(3);
		}
	      if (i > k) 
		k = i;
	      i++;
	    }
	  if (l > 30)
	    {
	      printf("Error, no device?\n");
	      outb(idle, port+2);
	      outb(0x00, port);
	      exit(4);
	    }

	  if (i == 0)
	    l++;
	  microdelay(2);
	}
      outb(prog_mode ^ inc_bit, port+2); /* XT1 = 1 */
      microdelay(5);
      outb(prog_mode, port+2); /* XT1 = 1 */
      microdelay(2);
      if (j % 64 == 0)
	printf(".");
    }
  outb(0xFF, port);
  nms(1);
  outb(prog_mode ^ vpp_bit, port+2); /* P3.2 = 0 !! */
  nms(1);
  outb(idle, port+2);
  outb(0x00, port);
  printf("\nWe are finished\n");
  exit(0);
}
